#____________________________________________________________________________#
#|                                                                          |#
#| This file contains procedures to create the bindings allowing the user   |#
#| to create a copy of the item that is currently selected.                 |#
#| 1. proc Copy_Shape_BP1 :                                                 |#
#|         Create the binding for <ButtonPress-1> in the selected item.     |#
#|         At this point a copy is created over the selected item.          |#
#| 2. proc Copy_Shape_B1M :                                                 |#
#|         Create the binding for <B1-Motion> in the canvas.                |#
#|         An outline can be dragged across the canvas.                     |#
#|         Activated if <BP-1> in the selected item has occured.            |#
#| 3. proc Copy_Shape_BR1 :                                                 |#
#|         Create the binding for <ButtonRelease-1> in the canvas.          |#
#|         If both the above bindings have been carried out, the copy is    |#
#|         moved to the new location.  If no motion has occured, then the   |#
#|         copy is deleted from the canvas.                                 |#
#|__________________________________________________________________________|#
#                                                                            #

#============================================================================#

proc Copy_Shape_BP1 {} {

# If no item is currently selected, then tags will be a null string.
# To avoid getting caught in that binding, display a message for the user,
# and then reset to the default bindings allowing a shape to be selected.
# If tags is not empty, then create the bindings allowing the selected shape
# to be cloned.
    set tags [.can.c1 gettags selected]

    if {$tags==""} {
# Set the message and display it in the message widget in the main window
        set msg "\nThere is currently no selected item.  Select one and \
            try again."
        Set_Message $msg
# Display the message for some time before resetting the default bindings
        after 2500 {
            uplevel #0 {set _CHOICE ""}
            Control ""
        }

    } else {
# "Copy_motion" is used so the action is only carried out if the correct
# sequence of events has been followed.  Start by setting to zero.
        uplevel #0 set Copy_motion 0

        .can.c1 bind selected <ButtonPress-1> {
# Set "Copy_motion" to one so the binding for dragging the outline is invoked.
            set Copy_motion 1

# Set relative position of cursor on canvas as soon as <BP-1>
# "current*" is used for "<B1-Motion>"
            set currentx [.can.c1 canvasx %x]
            set currenty [.can.c1 canvasy %y]

# "start*" is used to calculate the position for the copy.
            set startx $currentx
            set starty $currenty

            if {$_CHOICE=="COPY_SELECTED"} {
                set type [.can.c1 type selected]

# Get a list of tags for the item currently selected.
                set tagList [.can.c1 gettags selected]

# From the list of tags, get the attributes for the item, and determine 
# fill-style and colour.
                set attributes [lindex $tagList 1]
                set itemColour [lindex $attributes 1]
                set itemWidth [lindex $attributes 2]

# Create outline of selected item to be dragged across the canvas.
# Create a copy of item over original.  If correct motions carried out,
# just move copy else delete it.
                case $type in {
                    {circle oval rectangle} {
# Hold coordinates in a variable, and create an outline to be dragged
                        set edges [.can.c1 coords selected]
                        eval ".can.c1 create $type $edges \
                            -outline $itemColour -tags box"

# Create an exact copy of the item over the original
                        if {$_STYLE=="outline"} {
	                    eval ".can.c1 create $type $edges -fill {} \
                                -outline $itemColour \
                                -width $itemWidth -tags item$_NITEM"
                        } else {
                            eval ".can.c1 create $type $edges \
                                -fill $itemColour \
                                -outline {} -tags item$_NITEM"
                        }
                    }
# Do same if item is a polygon
                    {polygon line} {
                        set edges [.can.c1 coords selected]

                        eval ".can.c1 create line $edges -fill $itemColour \
                             -tags box"

                        if {$_STYLE=="outline"} {
                             eval ".can.c1 create line $edges \
                                 -fill $itemColour -width $itemWidth \
                                 -tags item$_NITEM"
                        } else {
                            eval ".can.c1 create polygon $edges \
                                -fill $itemColour \
                                -tags item$_NITEM"
                        }
                    }
                }
# End case
            }
        }
    }
}

#============================================================================#

proc Copy_Shape_B1M {} {

    bind .can.c1 <B1-Motion> {
        if {$Copy_motion} {
# Calulate new position of outline relative to old position
            set dx [expr ([.can.c1 canvasx %x]-$currentx)]
            set dy [expr ([.can.c1 canvasy %y]-$currenty)]

# Reset these values to hold new position for further movements
            set currentx [.can.c1 canvasx %x]
            set currenty [.can.c1 canvasy %y]

# Move the outline by calculated amount
            eval ".can.c1 move box $dx $dy"
        }
    }
}

#============================================================================#

proc Copy_Shape_BR1 {} {

    bind .can.c1 <ButtonRelease-1> {
# Delete the outline of the shape
        .can.c1 delete box

        if {$Copy_motion} {
# Set end position of the motion
            set endx [.can.c1 canvasx %x]
            set endy [.can.c1 canvasy %y]

# Calculate change in position
            set dx [expr ($endx-$startx)]
            set dy [expr ($endy-$starty)]

# Move the copy by the calculated amount
            .can.c1 move item$_NITEM $dx $dy

# The copy has the same attributes as the original, so add these
# to the list of tags
            .can.c1 addtag $attributes withtag item$_NITEM

# Give it "item" as a tag so it too can be selected
            .can.c1 addtag item withtag item$_NITEM

# Once the action has occured, reset variables to null-values
            set Copy_motion 0
            set _CHOICE ""

# "_NITEM" is used to give each item an individual tag.
# It is normally incremented by the "proc auto_select"
# Copies are not automatically selected, so '_NITEM' must be 
# incremented here if a copy is made 
            incr _NITEM

# Call procedure to reset default bindings
            Control ""
        } else {
# Else delete the previously made copy
            .can.c1 delete item$_NITEM
        }
    }
}
